package sdds;

/**
 * <p>Returned by an empty data structure.</p>
 * 
 * @author Stephen G. Ware
 * @param <O> the type of object the iterator will yield
 */
class EmptyIterator<O> implements java.util.Iterator<O> {

	/**
	 * <p>Always returns <tt>false</tt>.</p>
	 * 
	 * @return <tt>false</tt>
	 */
	public boolean hasNext(){
		return false;
	}
	
	/**
	 * <p>Always throws a {@link NoSuchElementException}.</p>
	 * 
	 * @return n/a
	 * @throws NoSuchElementException every time this method is called
	 */
	public O next(){
		throw new java.util.NoSuchElementException("No more elements exists for this iterator.");
	}
	
	/**
	 * <p>Always throws a {@link UnsupportedOperationException}.</p>
	 * 
	 * @throws UnsupportedOperationException every time this method is called
	 */
	public void remove(){
		throw new UnsupportedOperationException("This iterator does not support the remove() method.");
	}
}
