package sdds;

import sdds.List.Node;

/**
 * <p>Iterates through each element in a {@link List}.</p>
 * 
 * @author Stephen G. Ware
 * @param <O> the type of object the list holds
 */
class ListIterator<O> implements java.util.Iterator<O> {

	/** The current element */
	private Node<O> current;
	
	/**
	 * <p>Construct an iterator for a given {@link List}.</p>
	 * 
	 * @param list the list to iterate through
	 */
	ListIterator(List<O> list){
		current = list.last;
	}
	
	/**
	 * <p>Test whether the list has more elements.</p>
	 * 
	 * @return <tt>true</tt> if more elements exist in the list, <tt>false</tt> otherwise
	 */
	public boolean hasNext(){
		return current != null;
	}

	/**
	 * <p>Return the next element in the list</p>
	 * 
	 * @return the next elemement
	 */
	public O next(){
		if(current == null)
			throw new java.util.NoSuchElementException("No more elements exists for this iterator.");
		O next = current.content;
		current = current.next;
		return next;
	}
	
	/**
	 * <p>Always throws a {@link UnsupportedOperationException}.</p>
	 * 
	 * @throws UnsupportedOperationException every time the method is called
	 */
	public void remove(){
		throw new UnsupportedOperationException("This iterator does not support the remove() method.");
	}
}
