package unit_tests;

/**
 * <p>A object which implements {@link Object#equals} differently from <code>==</code> and {@link Object#hashCode} so as to created hash collisions.</p>
 * 
 * @author Stephen G. Ware
 */
public class Item {

	public final String name;
	
	private static int nextID = 0;
	public final int id;

	private final int hashCode;
	
	public Item(String name){
		this.name = name;
		this.id = nextID;
		nextID++;
		int hc = name.hashCode() / 3;
		if(hc % 2 == 0) hc *= -1;
		hashCode = hc;
	}
	
	public boolean equals(Object other){
		if(other instanceof Item)
			return name.equals(((Item) other).name);
		else return false;
	}
	
	public int hashCode(){
		return hashCode;
	}
	
	public String toString(){
		return "[item " + id + ": " + name + "]";
	}
	
	public static void resetID(){
		nextID = 0;
	}
}
