package unit_tests;

import sdds.List;
import java.util.Iterator;
import junit.framework.TestCase;

/**
 * <p>Tests common to all subclasses of {@link sdds.List}</p>
 * 
 * @author Stephen G. Ware
 */
public abstract class ListTest extends TestCase {

	protected static final Item a = new Item("a");
	protected static final Item b = new Item("b");
	protected static final Item c = new Item("c");
	protected static final Item d = new Item("d");
	protected static final Item a2 = new Item("a");
	protected static final Item b2 = new Item("b");
	protected static final Item c2 = new Item("c");
	protected static final Item[] fourItems = {a, b, c, d};
	
	public void testAddLength(List<Item> list){
		System.out.println("\n=== The length of an empty " + list.getClass().getSimpleName() + " shiould be 0; the length of a " + list.getClass().getSimpleName() + " with 1 element should be 1. ===");
		System.out.print("Begin with an empty list.  It should be length 0: ");
		assertEquals(list.length(), 0);
		System.out.println(list.length());
		System.out.print("Add " + a + ".  The new list should be of length 1: ");
		list = list.add(a);
		assertEquals(list.length(), 1);
		System.out.println(list.length());
	}
	
	public void testMultiAdd(List<Item> list){
		System.out.println("\n=== Multiple elements can be added to a " + list.getClass().getSimpleName() + " at once. ===");
		System.out.println("Begin with an empty list.  Add an array of 4 items: " + a + ", " + b + ", " + c + ", " + d);
		list = list.add(fourItems);
		System.out.print("The list should contain those four items:");
		Item gotten;
		for(int i=0; i<list.length(); i++){
			gotten = list.get(i);
			System.out.print(" " + gotten);
			assertSame(gotten, fourItems[i]);
		}
		System.out.println();
	}
	
	public void testGetLast(List<Item> list){
		System.out.println("\n=== The last element in a " + list.getClass().getSimpleName() + " should be the last element added. ===");
		System.out.print("Begin with an empty list.  Getting the last element should return null: ");
		Item gotten = list.getLast();
		if(gotten == null) System.out.println("null");
		assertNull(gotten);
		System.out.println("Add " + a + ", then " + b + ".");
		list = list.add(a);
		list = list.add(b);
		System.out.print("The last element should be " + b + ": ");
		gotten = list.getLast();
		System.out.println(gotten);
		assertSame(gotten, b);
	}
	
	public void testGet(List<Item> list){
		System.out.println("\n=== An element in a " + list.getClass().getSimpleName() + " can be retrieved by its index. ===");
		System.out.print("Begin with an empty list.  Getting element 0 should return null: ");
		Item gotten = list.get(0);
		if(gotten == null) System.out.println("null");
		assertNull(gotten);
		System.out.println("Add " + a + ", then " + b + ".");
		list = list.add(a);
		list = list.add(b);
		System.out.print("Element 0 should be " + a + ": ");
		gotten = list.get(0);
		System.out.println(gotten);
		assertSame(gotten, a);
		System.out.print("Element 1 should be " + b + ": ");
		gotten = list.get(1);
		assertSame(gotten, b);
		System.out.println(gotten);
	}
	
	public void testIterator(List<Item> list){
		System.out.println("\n=== The " + list.getClass().getSimpleName() + " iterator should yield all elements in the list. ===");
		System.out.println("Begin with an empty list.  Add " + a + ", then " + b + ".");
		list = list.add(a);
		list = list.add(b);
		System.out.print("Get an iterator for the list.  The iterator should throw an exception if we try to remove an element: ");
		Iterator<Item> iterator = list.iterator();
		try{
			iterator.remove();
			fail();
		}
		catch(UnsupportedOperationException ex){
			System.out.println(ex.getMessage());
		}
		System.out.print("Iterate through all elements: ");
		Item item;
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, b);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, a);
		System.out.print(item + ".\n");
		assertFalse(iterator.hasNext());
		System.out.print("The iterator should throw an exception if we try to get any more elements: ");
		try{
			iterator.next();
			fail();
		}
		catch(java.util.NoSuchElementException ex){
			System.out.println(ex.getMessage());
		}
	}
	
	public void testContains(List<Item> list){
		System.out.println("\n=== A " + list.getClass().getSimpleName() + " can report if it contains a given element. ===");
		System.out.println("Begin with an empty list.  Add " + a + ", then " + b + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(c);
		System.out.print("The list should contain " + a + ": ");
		assertTrue(list.contains(a));
		System.out.print("it does\nThe list should contain " + b + ": ");
		assertTrue(list.contains(b));
		System.out.print("it does\nThe list should contain " + c + ": ");
		assertTrue(list.contains(c));
		System.out.print("it does\nThe list should not contain " + d + ": ");
		assertFalse(list.contains(d));
		System.out.print("it does not\nThe list should not contain " + a2 + " because it is a different object: ");
		assertFalse(list.contains(a2));
		System.out.println("it does not");
	}
	
	public void testContainsEquals(List<Item> list){
		System.out.println("\n=== A " + list.getClass().getSimpleName() + " can report if it contains a given element using equals(). ===");
		System.out.println("Begin with an empty list.  Add " + a + ", then " + b + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(c);
		System.out.print("The list should contain " + a + ": ");
		assertTrue(list.containsEquals(a));
		System.out.print("it does\nThe list should contain " + b + ": ");
		assertTrue(list.containsEquals(b));
		System.out.print("it does\nThe list should contain " + c + ": ");
		assertTrue(list.containsEquals(c));
		System.out.print("it does\nThe list should not contain " + d + ": ");
		assertFalse(list.containsEquals(d));
		System.out.print("it does not\nThe list should contain " + a2 + ", which is a different object but equal to " + a + ": ");
		assertTrue(list.containsEquals(a2));
		System.out.println("it does");
	}
	
	public void testRemove(List<Item> list){
		System.out.println("\n=== Elements can be removed from a " + list.getClass().getSimpleName() + ". ===");
		System.out.print("Begin with an empty list.  Removing " + a + " should result in no change: ");
		assertSame(list, list.remove(a));
		System.out.println("no change\nAdd " + a + ", then " + b + ", then " + b + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(b);
		list = list.add(c);
		assertEquals(list.length(), 4);
		System.out.print("Remove " + b + ".  Remaining elements: ");
		list = list.remove(b);
		assertEquals(list.length(), 3);
		// The list should be: c b a
		Iterator<Item> iterator = list.iterator();
		Item item;
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, c);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, b);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, a);
		System.out.println(item);
		assertFalse(iterator.hasNext());
		System.out.print("Remove " + c + ".  Remaining elements: ");
		list = list.remove(c);
		assertEquals(list.length(), 2);
		// The list should be: b a
		iterator = list.iterator();
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, b);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, a);
		System.out.println(item);
	}
	
	public void testRemoveEquals(List<Item> list){
		System.out.println("\n=== Elements can be removed from a " + list.getClass().getSimpleName() + " using equals(). ===");
		System.out.print("Begin with an empty list.  Removing " + a + " using equals() should result in no change: ");
		assertSame(list, list.removeEquals(a2));
		System.out.println("no change\nAdd " + a + ", then " + b + ", then " + b + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(b);
		list = list.add(c);
		assertEquals(list.length(), 4);
		System.out.print("Remove " + b2 + " using equals().  Remaining elements: ");
		list = list.removeEquals(b2);
		assertEquals(list.length(), 3);
		// The list should be: c b a
		Iterator<Item> iterator = list.iterator();
		Item item;
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, c);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, b);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, a);
		System.out.println(item);
		assertFalse(iterator.hasNext());
		System.out.print("Remove " + c2 + " using equals().  Remaining elements: ");
		list = list.removeEquals(c2);
		assertEquals(list.length(), 2);
		// The list should be: b a
		iterator = list.iterator();
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, b);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, a);
		System.out.println(item);
	}
	
	public void testRemoveAll(List<Item> list){
		System.out.println("\n=== All instances of an element can be removed from a " + list.getClass().getSimpleName() + ". ===");
		System.out.print("Begin with an empty list.  Removing all instances of " + a + " should result in no change: ");
		assertSame(list, list.removeAll(a));
		System.out.println("no change\nAdd " + a + ", then " + b + ", then " + b + ", then " + c + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(b);
		list = list.add(c);
		list = list.add(c);
		assertEquals(list.length(), 5);
		System.out.print("Remove all instances of " + b + ".  Remaining elements: ");
		list = list.removeAll(b);
		assertEquals(list.length(), 3);
		// The list should be: c c a
		Iterator<Item> iterator = list.iterator();
		Item item;
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, c);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, c);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, a);
		System.out.println(item);
		assertFalse(iterator.hasNext());
		System.out.print("Remove all instances of " + c + ".  Remaining elements: ");
		list = list.removeAll(c);
		assertEquals(list.length(), 1);
		assertSame(list.getLast(), a);
		System.out.println(a);
	}
	
	public void testRemoveAllEquals(List<Item> list){
		System.out.println("\n=== All instances of an element can be removed from a " + list.getClass().getSimpleName() + " using equals(). ===");
		System.out.print("Begin with an empty list.  Removing all instances of " + a + " using equals() should result in no change: ");
		assertSame(list, list.removeAllEquals(a2));
		System.out.println("no change\nAdd " + a + ", then " + b + ", then " + b + ", then " + c + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(b);
		list = list.add(c);
		list = list.add(c);
		assertEquals(list.length(), 5);
		System.out.print("Remove all instances of " + b2 + " using equals().  Remaining elements: ");
		list = list.removeAllEquals(b2);
		assertEquals(list.length(), 3);
		// The list should be: c c a
		Iterator<Item> iterator = list.iterator();
		Item item;
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, c);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, c);
		System.out.print(item + " ");
		assertTrue(iterator.hasNext());
		item = iterator.next();
		assertSame(item, a);
		System.out.println(item);
		assertFalse(iterator.hasNext());
		System.out.print("Remove all instances of " + c2 + " using equals().  Remaining elements: ");
		list = list.removeAllEquals(c2);
		assertEquals(list.length(), 1);
		assertSame(list.getLast(), a);
		System.out.println(a);
	}
	
	public void testRemoveNonMember(List<Item> list){
		System.out.println("\n=== Removing an element not in a " + list.getClass().getSimpleName() + " should result in no change. ===");
		System.out.println("Begin with an empty list.  Add " + a + ", then " + b + ".");
		list = list.add(a);
		list = list.add(b);
		System.out.print("Removing " + c + " should result in no change: ");
		List<Item> removed = list.remove(c);
		assertEquals(list.length(), 2);
		if(removed != list) fail();
		System.out.println("no change");
		System.out.print("Removing " + c2 + " using equals should result in no change: ");
		removed = list.removeEquals(c2);
		assertEquals(list.length(), 2);
		if(removed != list) fail();
		System.out.println("no change");
	}
	
	public void testAddList(List<Item> list1, List<Item> list2){
		System.out.println("\n=== " + list1.getClass().getSimpleName() + "s can be concatenated. ===");
		System.out.println("Begin with two empty lists.  Add " + a + ", then " + b + " to the first.  Add " + c + ", then " + d + " to the second.");
		list1 = list1.add(a);
		list1 = list1.add(b);
		list2 = list2.add(c);
		list2 = list2.add(d);
		System.out.print("Add the second list onto the first:");
		list1 = list1.add(list2);
		assertEquals(list1.length(), 4);
		Item[] expected = new Item[4];
		expected[0] = d;
		expected[1] = c;
		expected[2] = b;
		expected[3] = a;
		Item item;
		int i = 0;
		for(Iterator<Item> iterator = list1.iterator(); iterator.hasNext();){
			item = iterator.next();
			assertSame(item, expected[i]);
			i++;
			System.out.print(" " + item);
		}
		System.out.println();
		System.out.print("Add " + a + " to the second list.  It should now be of length 3: ");
		list2 = list2.add(a);
		assertEquals(list2.length(), 3);
		System.out.println(list2.length());
		System.out.print("But the first list should still be of length 4: ");
		assertEquals(list1.length(), 4);
		System.out.println(list1.length());
	}
	
	public void testToArray(List<Item> list){
		System.out.println("\n=== A " + list.getClass().getSimpleName() + " can be converted to an array. ===");
		System.out.println("Begin with an empty list.  Add " + a + ", then " + b + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(c);
		System.out.print("Convert the list to an array: ");
		Item[] array = list.toArray(new Item[0]);
		System.out.print("[");
		assertSame(array[0], a);
		System.out.print(array[0] + ", ");
		assertSame(array[1], b);
		System.out.print(array[1] + ", ");
		assertSame(array[2], c);
		System.out.println(array[2] + "].");
	}
	
	public void testEquals(List<Item> list1, List<Item> list2){
		System.out.println("\n=== " + list1.getClass().getSimpleName() + "s can be compared using equals(). ===");
		assertNotSame(list1, list2);
		System.out.print("Begin with two empty lists.  The first list should not be equal to null: ");
		assertFalse(list1.equals(null));
		System.out.println("it is not\nAdd " + a + " and " + b + " to the first list.");
		list1 = list1.add(a);
		list1 = list1.add(b);
		System.out.print("The lists should not be equal: ");
		assertFalse(list1.equals(list2));
		System.out.println("they are not\nAdd " + c + " and " + b + " to the second list.");
		list2 = list2.add(c);
		list2 = list2.add(b);
		System.out.print("The lists should not be equal: ");
		assertFalse(list1.equals(list2));
		System.out.println("they are not\nRemove " + b + " and " + c + " from the second list.  Add " + a + " and " + b + " to the second list.");
		list2 = list2.remove(b);
		list2 = list2.remove(c);
		list2 = list2.add(a);
		list2 = list2.add(b);
		System.out.print("The lists should be equal: ");
		assertTrue(list1.equals(list2));
		System.out.println("they are");
	}
	
	public void testClone(List<Item> list){
		System.out.println("\n=== " + list.getClass().getSimpleName() + "s can be cloned. ===");
		System.out.println("Begin with an empty list.  Add " + a + ", then " + b + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(c);
		System.out.print("Clone the list.  The original and the clone should not be the same: ");
		List<Item> clone = list.clone();
		assertNotSame(list, clone);
		System.out.print("they are not\nThe original and the clone should be equal: ");
		assertEquals(list, clone);
		System.out.print("they are\nAdd " + d + " to the clone.  The original should remain unchanged: ");
		clone = clone.add(d);
		assertEquals(list.length(), 3);
		System.out.println("no change");
	}
	
	public void testToString(List<Item> list){
		System.out.println("\n=== Test toString() for " + list.getClass().getSimpleName() + ". ===");
		System.out.println("Begin with an empty list.  Add " + a + ", then " + b + ", then " + c + ".");
		list = list.add(a);
		list = list.add(b);
		list = list.add(c);
		System.out.println("Test the toString() method:\n");
		System.out.println(list.toString());
	}
}
