package unit_tests;

import sdds.SHashTable;

/**
 * <p>Tests for {@link sdds.SHashTable}</p>
 * 
 * @author Stephen G. Ware
 */
public class SHashTableTest extends HashTableTest {
	
	public void testArrayConstructor(){
		System.out.println("\n=== An SHashTable can be constructed from an array of keys and values. ===");
		System.out.println("Construct a hash table using an array of keys consisting of " + keys[0] + " to " + keys[keys.length - 1] + " and an array of values consisting of " + values[0] + " to " + values[values.length - 1] + ".");
		System.out.print("The hash table should have 100 mappings: ");
		SHashTable<Item, Item> hashTable = new SHashTable<Item, Item>(keys, values);
		assertEquals(hashTable.size(), 100);
		System.out.println(hashTable.size());
		System.out.print("Each key should map to its corresponding value: ");
		for(int i=0; i<keys.length; i++)
			assertSame(values[i], hashTable.get(keys[i]));
		System.out.println("they do\nConstruct a hash table using the same arrays, except that the first and second keys are both " + keys[0] + ".");
		System.out.print("The hash table should have 99 mappings: ");
		Item[] repeatKeys = keys.clone();
		repeatKeys[1] = repeatKeys[0];
		hashTable = new SHashTable<Item, Item>(repeatKeys, values);
		assertEquals(hashTable.size(), 99);
		System.out.println(hashTable.size());
		System.out.print("The key " + repeatKeys[0] + " should map to " + values[1] + ": ");
		assertSame(hashTable.get(repeatKeys[0]), values[1]);
		System.out.print("it does\nConstruct a hash table using an array of 100 keys and an array of 99 values.  An exception should be thrown: ");
		try{
			hashTable = new SHashTable<Item, Item>(keys, new Item[99]);
			fail();
		}
		catch(ArrayIndexOutOfBoundsException ex){
			System.out.println(ex.getMessage());
		}
	}
	
	public void testAddGet(){
		super.testAddGet(new SHashTable<Item, Item>());
	}
	
	public void test100AddGet(){
		super.test100AddGet(new SHashTable<Item, Item>());
	}
	
	public void test100AddGetEquals(){
		super.test100AddGetEquals(new SHashTable<Item, Item>());
	}
	
	public void testReplace(){
		super.testReplace(new SHashTable<Item, Item>());
	}
	
	public void testMultiAdd(){
		super.testMultiAdd(new SHashTable<Item, Item>());
	}
	
	public void test100MultiReplace(){
		super.test100MultiReplace(new SHashTable<Item, Item>());
	}
	
	public void testMultiAddEquals(){
		super.testMultiAddEquals(new SHashTable<Item, Item>());
	}
	
	public void testMultiAddDifferentSizeArrays(){
		super.testMultiAddDifferentSizeArrays(new SHashTable<Item, Item>());
	}
	
	public void testRemove(){
		super.testRemove(new SHashTable<Item, Item>());
	}

	public void testMultiRemove(){
		super.testMultiRemove(new SHashTable<Item, Item>());
	}
	
	public void testRemoveEquals(){
		super.testRemoveEquals(new SHashTable<Item, Item>());
	}
	
	public void testMultiRemoveEquals(){
		super.testMultiRemoveEquals(new SHashTable<Item, Item>());
	}
	
	public void testKeyIterator(){
		super.testKeyIterator(new SHashTable<Item, Item>());
	}
	
	public void testValueIterator(){
		super.testValueIterator(new SHashTable<Item, Item>());
	}
	
	public void testStatic(){
		System.out.println("\n=== Changing an SHashTable should not change the original. ===");
		SHashTable<Item, Item> hashTable1 = new SHashTable<Item, Item>();
		SHashTable<Item, Item> hashTable2;
		System.out.print("Begin with an empty static hash table.  It should have 0 entries: ");
		assertEquals(hashTable1.size(), 0);
		System.out.println(hashTable1.size());
		System.out.println("Add " + values[1] + " keyed on " + keys[1] + ".");
		hashTable2 = hashTable1.add(keys[1], values[1]);
		System.out.print("The original (first) hash table should still have 0 mappings: ");
		assertEquals(hashTable1.size(), 0);
		System.out.println(hashTable1.size());
		System.out.print("And getting " + keys[1] + " from the first hash table should return null: ");
		Item gotten = hashTable1.get(keys[1]);
		assertNull(gotten);
		System.out.println(gotten);
		System.out.print("The returned (second) hash table should have 1 mapping: ");
		assertEquals(hashTable2.size(), 1);
		System.out.println(hashTable2.size());
		System.out.print("And getting " + keys[1] + " from the second hash table should return " + values[1] + ": ");
		gotten = hashTable2.get(keys[1]);
		assertSame(gotten, values[1]);
		System.out.println(gotten);
		System.out.print("The hash tables should not be the same object: ");
		assertNotSame(hashTable1, hashTable2);
		System.out.println("they are not\nAdd " + values[2] + " keyed on " + keys[2] + " to the second hash table, discarding the returned hash table.");
		hashTable2.add(keys[2], values[2]);
		System.out.print("Getting " + keys[2] + " from the second hash table should return null: ");
		gotten = hashTable2.get(keys[2]);
		assertNull(gotten);
		System.out.println(gotten);
		System.out.print("Remove key " + keys[1] + " from the second hash table, discarding the returned hash table.  The second hash table should still have 1 mapping: ");
		hashTable2.remove(keys[1]);
		assertEquals(hashTable2.size(), 1);
		System.out.println(hashTable2.size());
	}
	
	public void testEquals(){
		super.testEquals(new SHashTable<Item, Item>(), new SHashTable<Item, Item>());
	}
	
	public void testClone(){
		super.testClone(new SHashTable<Item, Item>());
	}
	
	public void testToString(){
		super.testToString(new SHashTable<Item, Item>());
	}
}
